// **********************************************************************
//
// Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

#ifndef ICE_UTIL_UUID_H
#define ICE_UTIL_UUID_H

#include <IceUtil/Config.h>

namespace IceUtil
{

ICE_UTIL_API std::string generateUUID();

}

#endif
