// **********************************************************************
//
// Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

#ifndef ICEPY_SLICE_H
#define ICEPY_SLICE_H

#include <Config.h>

extern "C" PyObject* IcePy_loadSlice(PyObject*, PyObject*);
extern "C" PyObject* IcePy_compile(PyObject*, PyObject*);

#endif
