// **********************************************************************
//
// Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************
//
// Ice version 3.6.4
//
// <auto-generated>
//
// Generated from file `IceLocatorDiscovery.ice'
//
// Warning: do not edit this file.
//
// </auto-generated>
//

#ifndef __IceLocatorDiscovery_IceLocatorDiscovery_h__
#define __IceLocatorDiscovery_IceLocatorDiscovery_h__

#include <IceUtil/PushDisableWarnings.h>
#include <Ice/ProxyF.h>
#include <Ice/ObjectF.h>
#include <Ice/Exception.h>
#include <Ice/LocalObject.h>
#include <Ice/StreamHelpers.h>
#include <Ice/Proxy.h>
#include <Ice/GCObject.h>
#include <Ice/AsyncResult.h>
#include <Ice/Incoming.h>
#include <Ice/IncomingAsync.h>
#include <Ice/FactoryTableInit.h>
#include <IceUtil/ScopedArray.h>
#include <IceUtil/Optional.h>
#include <Ice/StreamF.h>
#include <Ice/Locator.h>
#include <IceUtil/UndefSysMacros.h>

#ifndef ICE_IGNORE_VERSION
#   if ICE_INT_VERSION / 100 != 306
#       error Ice version mismatch!
#   endif
#   if ICE_INT_VERSION % 100 > 50
#       error Beta header file detected
#   endif
#   if ICE_INT_VERSION % 100 < 4
#       error Ice patch level mismatch!
#   endif
#endif

namespace IceProxy
{

namespace IceLocatorDiscovery
{

class LookupReply;
void __read(::IceInternal::BasicStream*, ::IceInternal::ProxyHandle< ::IceProxy::IceLocatorDiscovery::LookupReply>&);
::IceProxy::Ice::Object* upCast(::IceProxy::IceLocatorDiscovery::LookupReply*);

class Lookup;
void __read(::IceInternal::BasicStream*, ::IceInternal::ProxyHandle< ::IceProxy::IceLocatorDiscovery::Lookup>&);
::IceProxy::Ice::Object* upCast(::IceProxy::IceLocatorDiscovery::Lookup*);

}

}

namespace IceLocatorDiscovery
{

class LookupReply;
::Ice::Object* upCast(::IceLocatorDiscovery::LookupReply*);
typedef ::IceInternal::Handle< ::IceLocatorDiscovery::LookupReply> LookupReplyPtr;
typedef ::IceInternal::ProxyHandle< ::IceProxy::IceLocatorDiscovery::LookupReply> LookupReplyPrx;
void __patch(LookupReplyPtr&, const ::Ice::ObjectPtr&);

class Lookup;
::Ice::Object* upCast(::IceLocatorDiscovery::Lookup*);
typedef ::IceInternal::Handle< ::IceLocatorDiscovery::Lookup> LookupPtr;
typedef ::IceInternal::ProxyHandle< ::IceProxy::IceLocatorDiscovery::Lookup> LookupPrx;
void __patch(LookupPtr&, const ::Ice::ObjectPtr&);

}

namespace IceAsync
{

}

namespace IceLocatorDiscovery
{

class Callback_LookupReply_foundLocator_Base : virtual public ::IceInternal::CallbackBase { };
typedef ::IceUtil::Handle< Callback_LookupReply_foundLocator_Base> Callback_LookupReply_foundLocatorPtr;

class Callback_Lookup_findLocator_Base : virtual public ::IceInternal::CallbackBase { };
typedef ::IceUtil::Handle< Callback_Lookup_findLocator_Base> Callback_Lookup_findLocatorPtr;

}

namespace IceProxy
{

namespace IceLocatorDiscovery
{

class LookupReply : virtual public ::IceProxy::Ice::Object
{
public:

    void foundLocator(const ::Ice::LocatorPrx& __p_prx)
    {
        foundLocator(__p_prx, 0);
    }
    void foundLocator(const ::Ice::LocatorPrx& __p_prx, const ::Ice::Context& __ctx)
    {
        foundLocator(__p_prx, &__ctx);
    }
#ifdef ICE_CPP11
    ::Ice::AsyncResultPtr
    begin_foundLocator(const ::Ice::LocatorPrx& __p_prx, const ::IceInternal::Function<void ()>& __response, const ::IceInternal::Function<void (const ::Ice::Exception&)>& __exception = ::IceInternal::Function<void (const ::Ice::Exception&)>(), const ::IceInternal::Function<void (bool)>& __sent = ::IceInternal::Function<void (bool)>())
    {
        return begin_foundLocator(__p_prx, 0, new ::IceInternal::Cpp11FnOnewayCallbackNC(__response, __exception, __sent));
    }
    ::Ice::AsyncResultPtr
    begin_foundLocator(const ::Ice::LocatorPrx& __p_prx, const ::IceInternal::Function<void (const ::Ice::AsyncResultPtr&)>& __completed, const ::IceInternal::Function<void (const ::Ice::AsyncResultPtr&)>& __sent = ::IceInternal::Function<void (const ::Ice::AsyncResultPtr&)>())
    {
        return begin_foundLocator(__p_prx, 0, ::Ice::newCallback(__completed, __sent), 0);
    }
    ::Ice::AsyncResultPtr
    begin_foundLocator(const ::Ice::LocatorPrx& __p_prx, const ::Ice::Context& __ctx, const ::IceInternal::Function<void ()>& __response, const ::IceInternal::Function<void (const ::Ice::Exception&)>& __exception = ::IceInternal::Function<void (const ::Ice::Exception&)>(), const ::IceInternal::Function<void (bool)>& __sent = ::IceInternal::Function<void (bool)>())
    {
        return begin_foundLocator(__p_prx, &__ctx, new ::IceInternal::Cpp11FnOnewayCallbackNC(__response, __exception, __sent), 0);
    }
    ::Ice::AsyncResultPtr
    begin_foundLocator(const ::Ice::LocatorPrx& __p_prx, const ::Ice::Context& __ctx, const ::IceInternal::Function<void (const ::Ice::AsyncResultPtr&)>& __completed, const ::IceInternal::Function<void (const ::Ice::AsyncResultPtr&)>& __sent = ::IceInternal::Function<void (const ::Ice::AsyncResultPtr&)>())
    {
        return begin_foundLocator(__p_prx, &__ctx, ::Ice::newCallback(__completed, __sent));
    }
#endif

    ::Ice::AsyncResultPtr begin_foundLocator(const ::Ice::LocatorPrx& __p_prx)
    {
        return begin_foundLocator(__p_prx, 0, ::IceInternal::__dummyCallback, 0);
    }

    ::Ice::AsyncResultPtr begin_foundLocator(const ::Ice::LocatorPrx& __p_prx, const ::Ice::Context& __ctx)
    {
        return begin_foundLocator(__p_prx, &__ctx, ::IceInternal::__dummyCallback, 0);
    }

    ::Ice::AsyncResultPtr begin_foundLocator(const ::Ice::LocatorPrx& __p_prx, const ::Ice::CallbackPtr& __del, const ::Ice::LocalObjectPtr& __cookie = 0)
    {
        return begin_foundLocator(__p_prx, 0, __del, __cookie);
    }

    ::Ice::AsyncResultPtr begin_foundLocator(const ::Ice::LocatorPrx& __p_prx, const ::Ice::Context& __ctx, const ::Ice::CallbackPtr& __del, const ::Ice::LocalObjectPtr& __cookie = 0)
    {
        return begin_foundLocator(__p_prx, &__ctx, __del, __cookie);
    }

    ::Ice::AsyncResultPtr begin_foundLocator(const ::Ice::LocatorPrx& __p_prx, const ::IceLocatorDiscovery::Callback_LookupReply_foundLocatorPtr& __del, const ::Ice::LocalObjectPtr& __cookie = 0)
    {
        return begin_foundLocator(__p_prx, 0, __del, __cookie);
    }

    ::Ice::AsyncResultPtr begin_foundLocator(const ::Ice::LocatorPrx& __p_prx, const ::Ice::Context& __ctx, const ::IceLocatorDiscovery::Callback_LookupReply_foundLocatorPtr& __del, const ::Ice::LocalObjectPtr& __cookie = 0)
    {
        return begin_foundLocator(__p_prx, &__ctx, __del, __cookie);
    }

    void end_foundLocator(const ::Ice::AsyncResultPtr&);
    
private:

    void foundLocator(const ::Ice::LocatorPrx&, const ::Ice::Context*);
    ::Ice::AsyncResultPtr begin_foundLocator(const ::Ice::LocatorPrx&, const ::Ice::Context*, const ::IceInternal::CallbackBasePtr&, const ::Ice::LocalObjectPtr& __cookie = 0);
    
public:
    
    ::IceInternal::ProxyHandle<LookupReply> ice_context(const ::Ice::Context& __context) const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_context(__context).get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_adapterId(const ::std::string& __id) const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_adapterId(__id).get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_endpoints(const ::Ice::EndpointSeq& __endpoints) const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_endpoints(__endpoints).get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_locatorCacheTimeout(int __timeout) const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_locatorCacheTimeout(__timeout).get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_connectionCached(bool __cached) const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_connectionCached(__cached).get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_endpointSelection(::Ice::EndpointSelectionType __est) const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_endpointSelection(__est).get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_secure(bool __secure) const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_secure(__secure).get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_preferSecure(bool __preferSecure) const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_preferSecure(__preferSecure).get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_router(const ::Ice::RouterPrx& __router) const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_router(__router).get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_locator(const ::Ice::LocatorPrx& __locator) const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_locator(__locator).get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_collocationOptimized(bool __co) const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_collocationOptimized(__co).get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_invocationTimeout(int __timeout) const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_invocationTimeout(__timeout).get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_twoway() const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_twoway().get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_oneway() const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_oneway().get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_batchOneway() const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_batchOneway().get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_datagram() const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_datagram().get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_batchDatagram() const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_batchDatagram().get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_compress(bool __compress) const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_compress(__compress).get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_timeout(int __timeout) const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_timeout(__timeout).get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_connectionId(const ::std::string& __id) const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_connectionId(__id).get());
    }
    
    ::IceInternal::ProxyHandle<LookupReply> ice_encodingVersion(const ::Ice::EncodingVersion& __v) const
    {
        return dynamic_cast<LookupReply*>(::IceProxy::Ice::Object::ice_encodingVersion(__v).get());
    }
    
    static const ::std::string& ice_staticId();

private: 
    virtual ::IceProxy::Ice::Object* __newInstance() const;
};

class Lookup : virtual public ::IceProxy::Ice::Object
{
public:

    void findLocator(const ::std::string& __p_instanceName, const ::IceLocatorDiscovery::LookupReplyPrx& __p_reply)
    {
        findLocator(__p_instanceName, __p_reply, 0);
    }
    void findLocator(const ::std::string& __p_instanceName, const ::IceLocatorDiscovery::LookupReplyPrx& __p_reply, const ::Ice::Context& __ctx)
    {
        findLocator(__p_instanceName, __p_reply, &__ctx);
    }
#ifdef ICE_CPP11
    ::Ice::AsyncResultPtr
    begin_findLocator(const ::std::string& __p_instanceName, const ::IceLocatorDiscovery::LookupReplyPrx& __p_reply, const ::IceInternal::Function<void ()>& __response, const ::IceInternal::Function<void (const ::Ice::Exception&)>& __exception = ::IceInternal::Function<void (const ::Ice::Exception&)>(), const ::IceInternal::Function<void (bool)>& __sent = ::IceInternal::Function<void (bool)>())
    {
        return begin_findLocator(__p_instanceName, __p_reply, 0, new ::IceInternal::Cpp11FnOnewayCallbackNC(__response, __exception, __sent));
    }
    ::Ice::AsyncResultPtr
    begin_findLocator(const ::std::string& __p_instanceName, const ::IceLocatorDiscovery::LookupReplyPrx& __p_reply, const ::IceInternal::Function<void (const ::Ice::AsyncResultPtr&)>& __completed, const ::IceInternal::Function<void (const ::Ice::AsyncResultPtr&)>& __sent = ::IceInternal::Function<void (const ::Ice::AsyncResultPtr&)>())
    {
        return begin_findLocator(__p_instanceName, __p_reply, 0, ::Ice::newCallback(__completed, __sent), 0);
    }
    ::Ice::AsyncResultPtr
    begin_findLocator(const ::std::string& __p_instanceName, const ::IceLocatorDiscovery::LookupReplyPrx& __p_reply, const ::Ice::Context& __ctx, const ::IceInternal::Function<void ()>& __response, const ::IceInternal::Function<void (const ::Ice::Exception&)>& __exception = ::IceInternal::Function<void (const ::Ice::Exception&)>(), const ::IceInternal::Function<void (bool)>& __sent = ::IceInternal::Function<void (bool)>())
    {
        return begin_findLocator(__p_instanceName, __p_reply, &__ctx, new ::IceInternal::Cpp11FnOnewayCallbackNC(__response, __exception, __sent), 0);
    }
    ::Ice::AsyncResultPtr
    begin_findLocator(const ::std::string& __p_instanceName, const ::IceLocatorDiscovery::LookupReplyPrx& __p_reply, const ::Ice::Context& __ctx, const ::IceInternal::Function<void (const ::Ice::AsyncResultPtr&)>& __completed, const ::IceInternal::Function<void (const ::Ice::AsyncResultPtr&)>& __sent = ::IceInternal::Function<void (const ::Ice::AsyncResultPtr&)>())
    {
        return begin_findLocator(__p_instanceName, __p_reply, &__ctx, ::Ice::newCallback(__completed, __sent));
    }
#endif

    ::Ice::AsyncResultPtr begin_findLocator(const ::std::string& __p_instanceName, const ::IceLocatorDiscovery::LookupReplyPrx& __p_reply)
    {
        return begin_findLocator(__p_instanceName, __p_reply, 0, ::IceInternal::__dummyCallback, 0);
    }

    ::Ice::AsyncResultPtr begin_findLocator(const ::std::string& __p_instanceName, const ::IceLocatorDiscovery::LookupReplyPrx& __p_reply, const ::Ice::Context& __ctx)
    {
        return begin_findLocator(__p_instanceName, __p_reply, &__ctx, ::IceInternal::__dummyCallback, 0);
    }

    ::Ice::AsyncResultPtr begin_findLocator(const ::std::string& __p_instanceName, const ::IceLocatorDiscovery::LookupReplyPrx& __p_reply, const ::Ice::CallbackPtr& __del, const ::Ice::LocalObjectPtr& __cookie = 0)
    {
        return begin_findLocator(__p_instanceName, __p_reply, 0, __del, __cookie);
    }

    ::Ice::AsyncResultPtr begin_findLocator(const ::std::string& __p_instanceName, const ::IceLocatorDiscovery::LookupReplyPrx& __p_reply, const ::Ice::Context& __ctx, const ::Ice::CallbackPtr& __del, const ::Ice::LocalObjectPtr& __cookie = 0)
    {
        return begin_findLocator(__p_instanceName, __p_reply, &__ctx, __del, __cookie);
    }

    ::Ice::AsyncResultPtr begin_findLocator(const ::std::string& __p_instanceName, const ::IceLocatorDiscovery::LookupReplyPrx& __p_reply, const ::IceLocatorDiscovery::Callback_Lookup_findLocatorPtr& __del, const ::Ice::LocalObjectPtr& __cookie = 0)
    {
        return begin_findLocator(__p_instanceName, __p_reply, 0, __del, __cookie);
    }

    ::Ice::AsyncResultPtr begin_findLocator(const ::std::string& __p_instanceName, const ::IceLocatorDiscovery::LookupReplyPrx& __p_reply, const ::Ice::Context& __ctx, const ::IceLocatorDiscovery::Callback_Lookup_findLocatorPtr& __del, const ::Ice::LocalObjectPtr& __cookie = 0)
    {
        return begin_findLocator(__p_instanceName, __p_reply, &__ctx, __del, __cookie);
    }

    void end_findLocator(const ::Ice::AsyncResultPtr&);
    
private:

    void findLocator(const ::std::string&, const ::IceLocatorDiscovery::LookupReplyPrx&, const ::Ice::Context*);
    ::Ice::AsyncResultPtr begin_findLocator(const ::std::string&, const ::IceLocatorDiscovery::LookupReplyPrx&, const ::Ice::Context*, const ::IceInternal::CallbackBasePtr&, const ::Ice::LocalObjectPtr& __cookie = 0);
    
public:
    
    ::IceInternal::ProxyHandle<Lookup> ice_context(const ::Ice::Context& __context) const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_context(__context).get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_adapterId(const ::std::string& __id) const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_adapterId(__id).get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_endpoints(const ::Ice::EndpointSeq& __endpoints) const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_endpoints(__endpoints).get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_locatorCacheTimeout(int __timeout) const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_locatorCacheTimeout(__timeout).get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_connectionCached(bool __cached) const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_connectionCached(__cached).get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_endpointSelection(::Ice::EndpointSelectionType __est) const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_endpointSelection(__est).get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_secure(bool __secure) const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_secure(__secure).get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_preferSecure(bool __preferSecure) const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_preferSecure(__preferSecure).get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_router(const ::Ice::RouterPrx& __router) const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_router(__router).get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_locator(const ::Ice::LocatorPrx& __locator) const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_locator(__locator).get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_collocationOptimized(bool __co) const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_collocationOptimized(__co).get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_invocationTimeout(int __timeout) const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_invocationTimeout(__timeout).get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_twoway() const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_twoway().get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_oneway() const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_oneway().get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_batchOneway() const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_batchOneway().get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_datagram() const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_datagram().get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_batchDatagram() const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_batchDatagram().get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_compress(bool __compress) const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_compress(__compress).get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_timeout(int __timeout) const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_timeout(__timeout).get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_connectionId(const ::std::string& __id) const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_connectionId(__id).get());
    }
    
    ::IceInternal::ProxyHandle<Lookup> ice_encodingVersion(const ::Ice::EncodingVersion& __v) const
    {
        return dynamic_cast<Lookup*>(::IceProxy::Ice::Object::ice_encodingVersion(__v).get());
    }
    
    static const ::std::string& ice_staticId();

private: 
    virtual ::IceProxy::Ice::Object* __newInstance() const;
};

}

}

namespace IceLocatorDiscovery
{

class LookupReply : virtual public ::Ice::Object
{
public:

    typedef LookupReplyPrx ProxyType;
    typedef LookupReplyPtr PointerType;

    virtual bool ice_isA(const ::std::string&, const ::Ice::Current& = ::Ice::Current()) const;
    virtual ::std::vector< ::std::string> ice_ids(const ::Ice::Current& = ::Ice::Current()) const;
    virtual const ::std::string& ice_id(const ::Ice::Current& = ::Ice::Current()) const;
    static const ::std::string& ice_staticId();

    virtual void foundLocator(const ::Ice::LocatorPrx&, const ::Ice::Current& = ::Ice::Current()) = 0;
    ::Ice::DispatchStatus ___foundLocator(::IceInternal::Incoming&, const ::Ice::Current&);

    virtual ::Ice::DispatchStatus __dispatch(::IceInternal::Incoming&, const ::Ice::Current&);

protected:
    virtual void __writeImpl(::IceInternal::BasicStream*) const;
    virtual void __readImpl(::IceInternal::BasicStream*);
    using ::Ice::Object::__writeImpl;
    using ::Ice::Object::__readImpl;
};

inline bool operator==(const LookupReply& l, const LookupReply& r)
{
    return static_cast<const ::Ice::Object&>(l) == static_cast<const ::Ice::Object&>(r);
}

inline bool operator<(const LookupReply& l, const LookupReply& r)
{
    return static_cast<const ::Ice::Object&>(l) < static_cast<const ::Ice::Object&>(r);
}

class Lookup : virtual public ::Ice::Object
{
public:

    typedef LookupPrx ProxyType;
    typedef LookupPtr PointerType;

    virtual bool ice_isA(const ::std::string&, const ::Ice::Current& = ::Ice::Current()) const;
    virtual ::std::vector< ::std::string> ice_ids(const ::Ice::Current& = ::Ice::Current()) const;
    virtual const ::std::string& ice_id(const ::Ice::Current& = ::Ice::Current()) const;
    static const ::std::string& ice_staticId();

    virtual void findLocator(const ::std::string&, const ::IceLocatorDiscovery::LookupReplyPrx&, const ::Ice::Current& = ::Ice::Current()) = 0;
    ::Ice::DispatchStatus ___findLocator(::IceInternal::Incoming&, const ::Ice::Current&);

    virtual ::Ice::DispatchStatus __dispatch(::IceInternal::Incoming&, const ::Ice::Current&);

protected:
    virtual void __writeImpl(::IceInternal::BasicStream*) const;
    virtual void __readImpl(::IceInternal::BasicStream*);
    using ::Ice::Object::__writeImpl;
    using ::Ice::Object::__readImpl;
};

inline bool operator==(const Lookup& l, const Lookup& r)
{
    return static_cast<const ::Ice::Object&>(l) == static_cast<const ::Ice::Object&>(r);
}

inline bool operator<(const Lookup& l, const Lookup& r)
{
    return static_cast<const ::Ice::Object&>(l) < static_cast<const ::Ice::Object&>(r);
}

}

namespace IceLocatorDiscovery
{

template<class T>
class CallbackNC_LookupReply_foundLocator : public Callback_LookupReply_foundLocator_Base, public ::IceInternal::OnewayCallbackNC<T>
{
public:

    typedef IceUtil::Handle<T> TPtr;

    typedef void (T::*Exception)(const ::Ice::Exception&);
    typedef void (T::*Sent)(bool);
    typedef void (T::*Response)();

    CallbackNC_LookupReply_foundLocator(const TPtr& obj, Response cb, Exception excb, Sent sentcb)
        : ::IceInternal::OnewayCallbackNC<T>(obj, cb, excb, sentcb)
    {
    }
};

template<class T> Callback_LookupReply_foundLocatorPtr
newCallback_LookupReply_foundLocator(const IceUtil::Handle<T>& instance, void (T::*cb)(), void (T::*excb)(const ::Ice::Exception&), void (T::*sentcb)(bool) = 0)
{
    return new CallbackNC_LookupReply_foundLocator<T>(instance, cb, excb, sentcb);
}

template<class T> Callback_LookupReply_foundLocatorPtr
newCallback_LookupReply_foundLocator(const IceUtil::Handle<T>& instance, void (T::*excb)(const ::Ice::Exception&), void (T::*sentcb)(bool) = 0)
{
    return new CallbackNC_LookupReply_foundLocator<T>(instance, 0, excb, sentcb);
}

template<class T> Callback_LookupReply_foundLocatorPtr
newCallback_LookupReply_foundLocator(T* instance, void (T::*cb)(), void (T::*excb)(const ::Ice::Exception&), void (T::*sentcb)(bool) = 0)
{
    return new CallbackNC_LookupReply_foundLocator<T>(instance, cb, excb, sentcb);
}

template<class T> Callback_LookupReply_foundLocatorPtr
newCallback_LookupReply_foundLocator(T* instance, void (T::*excb)(const ::Ice::Exception&), void (T::*sentcb)(bool) = 0)
{
    return new CallbackNC_LookupReply_foundLocator<T>(instance, 0, excb, sentcb);
}

template<class T, typename CT>
class Callback_LookupReply_foundLocator : public Callback_LookupReply_foundLocator_Base, public ::IceInternal::OnewayCallback<T, CT>
{
public:

    typedef IceUtil::Handle<T> TPtr;

    typedef void (T::*Exception)(const ::Ice::Exception& , const CT&);
    typedef void (T::*Sent)(bool , const CT&);
    typedef void (T::*Response)(const CT&);

    Callback_LookupReply_foundLocator(const TPtr& obj, Response cb, Exception excb, Sent sentcb)
        : ::IceInternal::OnewayCallback<T, CT>(obj, cb, excb, sentcb)
    {
    }
};

template<class T, typename CT> Callback_LookupReply_foundLocatorPtr
newCallback_LookupReply_foundLocator(const IceUtil::Handle<T>& instance, void (T::*cb)(const CT&), void (T::*excb)(const ::Ice::Exception&, const CT&), void (T::*sentcb)(bool, const CT&) = 0)
{
    return new Callback_LookupReply_foundLocator<T, CT>(instance, cb, excb, sentcb);
}

template<class T, typename CT> Callback_LookupReply_foundLocatorPtr
newCallback_LookupReply_foundLocator(const IceUtil::Handle<T>& instance, void (T::*excb)(const ::Ice::Exception&, const CT&), void (T::*sentcb)(bool, const CT&) = 0)
{
    return new Callback_LookupReply_foundLocator<T, CT>(instance, 0, excb, sentcb);
}

template<class T, typename CT> Callback_LookupReply_foundLocatorPtr
newCallback_LookupReply_foundLocator(T* instance, void (T::*cb)(const CT&), void (T::*excb)(const ::Ice::Exception&, const CT&), void (T::*sentcb)(bool, const CT&) = 0)
{
    return new Callback_LookupReply_foundLocator<T, CT>(instance, cb, excb, sentcb);
}

template<class T, typename CT> Callback_LookupReply_foundLocatorPtr
newCallback_LookupReply_foundLocator(T* instance, void (T::*excb)(const ::Ice::Exception&, const CT&), void (T::*sentcb)(bool, const CT&) = 0)
{
    return new Callback_LookupReply_foundLocator<T, CT>(instance, 0, excb, sentcb);
}

template<class T>
class CallbackNC_Lookup_findLocator : public Callback_Lookup_findLocator_Base, public ::IceInternal::OnewayCallbackNC<T>
{
public:

    typedef IceUtil::Handle<T> TPtr;

    typedef void (T::*Exception)(const ::Ice::Exception&);
    typedef void (T::*Sent)(bool);
    typedef void (T::*Response)();

    CallbackNC_Lookup_findLocator(const TPtr& obj, Response cb, Exception excb, Sent sentcb)
        : ::IceInternal::OnewayCallbackNC<T>(obj, cb, excb, sentcb)
    {
    }
};

template<class T> Callback_Lookup_findLocatorPtr
newCallback_Lookup_findLocator(const IceUtil::Handle<T>& instance, void (T::*cb)(), void (T::*excb)(const ::Ice::Exception&), void (T::*sentcb)(bool) = 0)
{
    return new CallbackNC_Lookup_findLocator<T>(instance, cb, excb, sentcb);
}

template<class T> Callback_Lookup_findLocatorPtr
newCallback_Lookup_findLocator(const IceUtil::Handle<T>& instance, void (T::*excb)(const ::Ice::Exception&), void (T::*sentcb)(bool) = 0)
{
    return new CallbackNC_Lookup_findLocator<T>(instance, 0, excb, sentcb);
}

template<class T> Callback_Lookup_findLocatorPtr
newCallback_Lookup_findLocator(T* instance, void (T::*cb)(), void (T::*excb)(const ::Ice::Exception&), void (T::*sentcb)(bool) = 0)
{
    return new CallbackNC_Lookup_findLocator<T>(instance, cb, excb, sentcb);
}

template<class T> Callback_Lookup_findLocatorPtr
newCallback_Lookup_findLocator(T* instance, void (T::*excb)(const ::Ice::Exception&), void (T::*sentcb)(bool) = 0)
{
    return new CallbackNC_Lookup_findLocator<T>(instance, 0, excb, sentcb);
}

template<class T, typename CT>
class Callback_Lookup_findLocator : public Callback_Lookup_findLocator_Base, public ::IceInternal::OnewayCallback<T, CT>
{
public:

    typedef IceUtil::Handle<T> TPtr;

    typedef void (T::*Exception)(const ::Ice::Exception& , const CT&);
    typedef void (T::*Sent)(bool , const CT&);
    typedef void (T::*Response)(const CT&);

    Callback_Lookup_findLocator(const TPtr& obj, Response cb, Exception excb, Sent sentcb)
        : ::IceInternal::OnewayCallback<T, CT>(obj, cb, excb, sentcb)
    {
    }
};

template<class T, typename CT> Callback_Lookup_findLocatorPtr
newCallback_Lookup_findLocator(const IceUtil::Handle<T>& instance, void (T::*cb)(const CT&), void (T::*excb)(const ::Ice::Exception&, const CT&), void (T::*sentcb)(bool, const CT&) = 0)
{
    return new Callback_Lookup_findLocator<T, CT>(instance, cb, excb, sentcb);
}

template<class T, typename CT> Callback_Lookup_findLocatorPtr
newCallback_Lookup_findLocator(const IceUtil::Handle<T>& instance, void (T::*excb)(const ::Ice::Exception&, const CT&), void (T::*sentcb)(bool, const CT&) = 0)
{
    return new Callback_Lookup_findLocator<T, CT>(instance, 0, excb, sentcb);
}

template<class T, typename CT> Callback_Lookup_findLocatorPtr
newCallback_Lookup_findLocator(T* instance, void (T::*cb)(const CT&), void (T::*excb)(const ::Ice::Exception&, const CT&), void (T::*sentcb)(bool, const CT&) = 0)
{
    return new Callback_Lookup_findLocator<T, CT>(instance, cb, excb, sentcb);
}

template<class T, typename CT> Callback_Lookup_findLocatorPtr
newCallback_Lookup_findLocator(T* instance, void (T::*excb)(const ::Ice::Exception&, const CT&), void (T::*sentcb)(bool, const CT&) = 0)
{
    return new Callback_Lookup_findLocator<T, CT>(instance, 0, excb, sentcb);
}

}

#include <IceUtil/PopDisableWarnings.h>
#endif
