// **********************************************************************
//
// Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************
//
// Ice version 3.6.4
//
// <auto-generated>
//
// Generated from file `EndpointInfo.ice'
//
// Warning: do not edit this file.
//
// </auto-generated>
//

#ifndef __IceSSL_EndpointInfo_h__
#define __IceSSL_EndpointInfo_h__

#include <IceUtil/PushDisableWarnings.h>
#include <Ice/ProxyF.h>
#include <Ice/ObjectF.h>
#include <Ice/Exception.h>
#include <Ice/LocalObject.h>
#include <Ice/StreamHelpers.h>
#include <IceUtil/ScopedArray.h>
#include <IceUtil/Optional.h>
#include <Ice/Endpoint.h>
#include <IceUtil/UndefSysMacros.h>

#ifndef ICE_IGNORE_VERSION
#   if ICE_INT_VERSION / 100 != 306
#       error Ice version mismatch!
#   endif
#   if ICE_INT_VERSION % 100 > 50
#       error Beta header file detected
#   endif
#   if ICE_INT_VERSION % 100 < 4
#       error Ice patch level mismatch!
#   endif
#endif

#ifndef ICE_SSL_API
#   ifdef ICE_SSL_API_EXPORTS
#       define ICE_SSL_API ICE_DECLSPEC_EXPORT
#   elif defined(ICE_STATIC_LIBS)
#       define ICE_SSL_API /**/
#   else
#       define ICE_SSL_API ICE_DECLSPEC_IMPORT
#   endif
#endif

namespace IceSSL
{

class EndpointInfo;
ICE_SSL_API ::Ice::LocalObject* upCast(::IceSSL::EndpointInfo*);
typedef ::IceInternal::Handle< ::IceSSL::EndpointInfo> EndpointInfoPtr;

class WSSEndpointInfo;
ICE_SSL_API ::Ice::LocalObject* upCast(::IceSSL::WSSEndpointInfo*);
typedef ::IceInternal::Handle< ::IceSSL::WSSEndpointInfo> WSSEndpointInfoPtr;

}

namespace IceSSL
{

const ::Ice::Short EndpointType = ::Ice::SSLEndpointType;

}

namespace IceSSL
{

class ICE_SSL_API EndpointInfo : public ::Ice::IPEndpointInfo
{
public:

    typedef EndpointInfoPtr PointerType;

    EndpointInfo()
    {
    }

    EndpointInfo(::Ice::Int __ice_timeout, bool __ice_compress, const ::std::string& __ice_host, ::Ice::Int __ice_port, const ::std::string& __ice_sourceAddress) :
        ::Ice::IPEndpointInfo(__ice_timeout, __ice_compress, __ice_host, __ice_port, __ice_sourceAddress)
    {
    }

};

inline bool operator==(const EndpointInfo& l, const EndpointInfo& r)
{
    return static_cast<const ::Ice::LocalObject&>(l) == static_cast<const ::Ice::LocalObject&>(r);
}

inline bool operator<(const EndpointInfo& l, const EndpointInfo& r)
{
    return static_cast<const ::Ice::LocalObject&>(l) < static_cast<const ::Ice::LocalObject&>(r);
}

class ICE_SSL_API WSSEndpointInfo : public ::IceSSL::EndpointInfo
{
public:

    typedef WSSEndpointInfoPtr PointerType;

    WSSEndpointInfo()
    {
    }

    WSSEndpointInfo(::Ice::Int __ice_timeout, bool __ice_compress, const ::std::string& __ice_host, ::Ice::Int __ice_port, const ::std::string& __ice_sourceAddress, const ::std::string& __ice_resource) :
        ::IceSSL::EndpointInfo(__ice_timeout, __ice_compress, __ice_host, __ice_port, __ice_sourceAddress),
        resource(__ice_resource)
    {
    }


public:

    ::std::string resource;
};

inline bool operator==(const WSSEndpointInfo& l, const WSSEndpointInfo& r)
{
    return static_cast<const ::Ice::LocalObject&>(l) == static_cast<const ::Ice::LocalObject&>(r);
}

inline bool operator<(const WSSEndpointInfo& l, const WSSEndpointInfo& r)
{
    return static_cast<const ::Ice::LocalObject&>(l) < static_cast<const ::Ice::LocalObject&>(r);
}

}

#include <IceUtil/PopDisableWarnings.h>
#endif
