// **********************************************************************
//
// Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

#ifndef ICE_REGISTER_PLUGINS_INIT_H
#define ICE_REGISTER_PLUGINS_INIT_H

namespace IceInternal
{

class RegisterPluginsInit
{
public:

    RegisterPluginsInit();
};

}

#endif
